@@ -33,6 +33,8 @@ typedef struct Plugin Plugin;
 typedef struct MidiEvents MidiEvents;
 typedef struct Fader Fader;
 typedef struct SampleProcessor SampleProcessor;
+typedef struct PassthroughProcessor
+  PassthroughProcessor;
 
 /**
  * @addtogroup audio
@@ -80,6 +82,7 @@ typedef enum PortOwnerType
   PORT_OWNER_TYPE_BACKEND,
   PORT_OWNER_TYPE_PLUGIN,
   PORT_OWNER_TYPE_TRACK,
+  PORT_OWNER_TYPE_PREFADER,
   PORT_OWNER_TYPE_FADER,
   PORT_OWNER_TYPE_SAMPLE_PROCESSOR,
 } PortOwnerType;
@@ -202,6 +205,33 @@ typedef struct Port
    * 1.f. They correspond to each destination.
    */
   float               multipliers[MAX_DESTINATIONS];
+
+  /**
+   * These indicate whether the destination Port
+   * can be removed or the multiplier edited by the
+   * user.
+   *
+   * These are ignored when connecting things
+   * internally and are only used to deter the user
+   * from breaking necessary connections.
+   *
+   * 0 == unlocked, 1 == locked.
+   */
+  int                 dest_locked[MAX_DESTINATIONS];
+
+  /**
+   * These indicate whether the connection is
+   * enabled.
+   *
+   * The user can disable port connections only if
+   * they are not locked.
+   *
+   * 0 == disabled (disconnected),
+   * 1 == enabled (connected).
+   */
+  int                 dest_enabled[MAX_DESTINATIONS];
+
+  /** Counters. */
   int                 num_srcs;
   int                 num_dests;
 
@@ -481,6 +511,76 @@ port_new_with_data (
 StereoPorts *
 stereo_ports_new (Port * l, Port * r);
 
+/**
+ * Receives MIDI data from the port's exposed
+ * JACK port (if any) into the port.
+ */
+void
+port_receive_midi_events_from_jack (
+  Port * port,
+  int    start_frames,
+  int    nframes);
+
+/**
+ * Receives audio data from the port's exposed
+ * JACK port (if any) into the port.
+ */
+void
+port_receive_audio_data_from_jack (
+  Port * port,
+  int    start_frames,
+  int    nframes);
+
+/**
+ * Sends MIDI data from the port to its exposed
+ * JACK port (if any).
+ */
+void
+port_send_midi_events_to_jack (
+  Port * port,
+  int    start_frames,
+  int    nframes);
+
+/**
+ * Sends audio data from the port to its exposed
+ * JACK port (if any).
+ */
+void
+port_send_audio_data_to_jack (
+  Port *      port,
+  int         start_frames,
+  int         nframes);
+
+/**
+ * Sums the inputs coming in from JACK, before the
+ * port is processed.
+ */
+void
+port_sum_data_from_jack (
+  Port * port,
+  const int start_frames,
+  const int nframes);
+
+/**
+ * Sends the port data to JACK, after the port
+ * is processed.
+ */
+void
+port_send_data_to_jack (
+  Port * self,
+  const int start_frame,
+  const int nframes);
+
+/**
+ * Returns a full designation of the port in the
+ * format "Track/Port" or "Track/Plugin/Port".
+ *
+ * Must be free'd.
+ */
+char *
+port_get_full_designation (
+  Port * self);
+
 /**
  * Gathers all ports in the project and puts them
  * in the given array and size.
@@ -538,11 +638,27 @@ port_get_multiplier_by_index (
 void
 port_free (Port * port);
 
+#ifdef HAVE_JACK
+/**
+ * Sets whether to expose the port to JACk and
+ * exposes it or removes it from JACK.
+ */
+void
+port_set_expose_to_jack (
+  Port * self,
+  int    expose);
+#endif
+
 /**
  * Connets src to dest.
+ *
+ * @param locked Lock the connection or not.
  */
 int
-port_connect (Port * src, Port * dest);
+port_connect (
+  Port * src,
+  Port * dest,
+  const int locked);
 
 /**
  * Disconnects src from dest.
@@ -620,6 +736,14 @@ port_set_owner_fader (
   Port *    port,
   Fader *   fader);
 
+/**
+ * Sets the owner fader & its ID.
+ */
+void
+port_set_owner_prefader (
+  Port *                 port,
+  PassthroughProcessor * fader);
+
 /**
  * Sets the owner plugin & its ID.
  */
@@ -628,8 +752,22 @@ port_set_owner_plugin (
   Port *   port,
   Plugin * pl);
 
+/**
+ * Returns if the two ports are connected or not.
+ */
 int
-ports_connected (Port * src, Port * dest);
+ports_connected (
+  Port * src, Port * dest);
+
+/**
+ * Returns whether the Port's can be connected (if
+ * the connection will be valid and won't break the
+ * acyclicity of the graph).
+ */
+int
+ports_can_be_connected (
+  const Port * src,
+  const Port *dest);
 
 /**
  * Disconnects all the given ports.
