@@ -651,6 +651,14 @@ region_set_clip_start_pos (
 int
 region_is_selected (Region * self);
 
+/**
+ * Updates the frames of each position in each child
+ * of the region recursively.
+ */
+void
+region_update_frames (
+  Region * self);
+
 /**
  * Returns if Region is (should be) visible.
  */
