@@ -17,7 +17,10 @@
  * along with Zrythm.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-/** \file
+/**
+ * \file
+ *
+ * Routing graph.
  */
 
 #ifndef __AUDIO_ROUTING_H__
@@ -26,6 +29,17 @@
 #include <pthread.h>
 #include "zix/sem.h"
 
+typedef struct GraphNode GraphNode;
+typedef struct Graph Graph;
+typedef struct PassthroughProcessor
+  PassthroughProcessor;
+
+/**
+ * @addtogroup audio
+ *
+ * @{
+ */
+
 /**
  * Graph nodes can be either ports or processors.
  *
@@ -39,12 +53,12 @@ typedef enum GraphNodeType
   ROUTE_NODE_TYPE_PLUGIN,
   /** Fader/pan processor. */
   ROUTE_NODE_TYPE_FADER,
+  /** Pre-Fader passthrough processor. */
+  ROUTE_NODE_TYPE_PREFADER,
   /** Sample processor. */
   ROUTE_NODE_TYPE_SAMPLE_PROCESSOR,
 } GraphNodeType;
 
-typedef struct GraphNode GraphNode;
-typedef struct Graph Graph;
 typedef struct GraphNode
 {
   int           id;
@@ -59,7 +73,10 @@ typedef struct GraphNode
   GraphNode **  childnodes;
   int           n_childnodes;
 
+  /** Incoming node count. */
   volatile gint refcount;
+
+  /** Initial incoming node count. */
   gint          init_refcount;
 
   /** Used when creating the graph so we can
@@ -75,6 +92,9 @@ typedef struct GraphNode
   /** Fader, if fader. */
   Fader *       fader;
 
+  /** Pre-Fader, if prefader node. */
+  PassthroughProcessor * prefader;
+
   /** Sample processor, if sample processor. */
   SampleProcessor * sample_processor;
 
@@ -197,13 +217,19 @@ typedef struct Router
 } Router;
 
 /**
- * Creates a graph.
+ * Returns a new graph, or NULL if the graph is
+ * invalid.
+ *
+ * Optionally adds a new connection for the given
+ * src and dest ports.
  *
  * Should be used every time the graph is changed.
  */
 Graph *
 graph_new (
-  Router * router);
+  Router * router,
+  const Port *   src,
+  const Port *   dest);
 
 void
 router_init (
@@ -239,4 +265,15 @@ void
 graph_destroy (
   Graph * graph);
 
+/**
+ * Frees the graph and its members.
+ */
+void
+graph_free (
+  Graph * self);
+
+/**
+ * @}
+ */
+
 #endif
