@@ -172,6 +172,8 @@ _breeze_icons = [
   'edit-select',
   'plugins',
   'hand',
+  'offline',
+  'online',
   'window-maximize',
   ]
 for cat in ['actions', 'animations', 'applets', 'apps', 'categories', 'devices', 'emblems', 'emotes', 'mimetypes', 'places', 'preferences', 'status']:
