@@ -972,6 +972,23 @@ activate_create_ins_track (GSimpleAction *action,
   undo_manager_perform (UNDO_MANAGER, ua);
 }
 
+void
+activate_create_midi_track (
+  GSimpleAction *action,
+  GVariant      *variant,
+  gpointer       user_data)
+{
+  UndoableAction * ua =
+    create_tracks_action_new (
+      TRACK_TYPE_MIDI,
+      NULL,
+      NULL,
+      TRACKLIST->num_tracks,
+      1);
+
+  undo_manager_perform (UNDO_MANAGER, ua);
+}
+
 void
 activate_create_bus_track (GSimpleAction *action,
                   GVariant      *variant,
@@ -1194,3 +1211,30 @@ activate_quantize_options (
     }
   g_message ("quantize opts");
 }
+
+void
+activate_set_timebase_master (
+  GSimpleAction *action,
+  GVariant      *variant,
+  gpointer       user_data)
+{
+  g_message ("set time base master");
+}
+
+void
+activate_set_transport_client (
+  GSimpleAction *action,
+  GVariant      *variant,
+  gpointer       user_data)
+{
+  g_message ("set transport client");
+}
+
+void
+activate_unlink_jack_transport (
+  GSimpleAction *action,
+  GVariant      *variant,
+  gpointer       user_data)
+{
+  g_message ("unlink jack transport");
+}
