@@ -53,21 +53,43 @@
 
 #include <gtk/gtk.h>
 
-/**
- * Connects a plugin that has at least one MIDI
- * input port.
- */
 static inline void
-connect_midi_ins (
+connect_midi_in_to_midi_out (
+  Channel * ch)
+{
+  port_connect (ch->midi_in, ch->midi_out, 1);
+}
+
+static inline void
+connect_piano_roll_to_midi_out (
+  Channel * ch)
+{
+  port_connect (ch->piano_roll, ch->midi_out, 1);
+}
+
+static inline void
+disconnect_midi_in_from_midi_out (
+  Channel * ch)
+{
+  port_disconnect (ch->midi_in, ch->midi_out);
+}
+
+static inline void
+disconnect_piano_roll_from_midi_out (
+  Channel * ch)
+{
+  port_disconnect (ch->piano_roll, ch->midi_out);
+}
+
+static inline void
+connect_piano_roll_to_pl (
   Channel * ch,
-  int       pos,
-  Plugin *  pl)
+  Plugin * pl)
 {
   int i;
   Port * in_port;
 
-  /* Connect MIDI port and piano roll to the
-   * plugin */
+  /* Connect piano roll to the plugin */
   for (i = 0; i < pl->num_in_ports; i++)
     {
       in_port = pl->in_ports[i];
@@ -75,27 +97,41 @@ connect_midi_ins (
           in_port->identifier.flow == FLOW_INPUT)
         {
           port_connect (
-            ch->midi_in,
-            in_port);
-          if (ch->type == CT_MIDI &&
-              pos == 0)
-            {
-              port_connect (
-                ch->piano_roll,
-                in_port);
-            }
+            ch->piano_roll, in_port, 1);
         }
     }
 }
 
 /**
- * Disconnects a plugin that has at least one MIDI
+ * Connects a plugin that has at least one MIDI
  * input port.
  */
 static inline void
-disconnect_midi_ins (
+connect_midi_in_to_pl (
+  Channel * ch,
+  Plugin *  pl)
+{
+  int i;
+  Port * in_port;
+
+  /* Connect MIDI port to the plugin */
+  for (i = 0; i < pl->num_in_ports; i++)
+    {
+      in_port = pl->in_ports[i];
+      if (in_port->identifier.type == TYPE_EVENT &&
+          in_port->identifier.flow == FLOW_INPUT)
+        {
+          port_connect (ch->midi_in, in_port, 1);
+        }
+    }
+}
+
+/**
+ * Disconnects MIDI in from a plugin.
+ */
+static inline void
+disconnect_midi_in_from_pl (
   Channel * ch,
-  int       pos,
   Plugin *  pl)
 {
   int i;
@@ -112,16 +148,77 @@ disconnect_midi_ins (
           if (ports_connected (
                 ch->midi_in, in_port))
             port_disconnect (
-              ch->midi_in,
-              in_port);
-          if (ch->type == CT_MIDI &&
-              pos == 0)
+              ch->midi_in, in_port);
+        }
+    }
+}
+
+/**
+ * Disconnects piano roll from a plugin.
+ */
+static inline void
+disconnect_piano_roll_from_pl (
+  Channel * ch,
+  Plugin *  pl)
+{
+  int i;
+  Port * in_port;
+
+  /* Disconnect MIDI port and piano roll to the
+   * plugin */
+  for (i = 0; i < pl->num_in_ports; i++)
+    {
+      in_port = pl->in_ports[i];
+      if (in_port->identifier.type == TYPE_EVENT &&
+          in_port->identifier.flow == FLOW_INPUT)
+        {
+          if (ports_connected (
+                ch->midi_in, in_port))
+            port_disconnect (
+              ch->piano_roll, in_port);
+        }
+    }
+}
+
+static inline void
+connect_pl_to_midi_out (
+  Plugin * pl,
+  Channel * ch)
+{
+  int i;
+  Port * out_port;
+
+  for (i = 0; i < pl->num_out_ports; i++)
+    {
+      out_port = pl->out_ports[i];
+      if (out_port->identifier.type == TYPE_EVENT &&
+          out_port->identifier.flow == FLOW_OUTPUT)
+        {
+          port_connect (
+            out_port, ch->midi_out, 1);
+        }
+    }
+}
+
+static inline void
+disconnect_pl_from_midi_out (
+  Plugin * pl,
+  Channel * ch)
+{
+  int i;
+  Port * out_port;
+
+  for (i = 0; i < pl->num_out_ports; i++)
+    {
+      out_port = pl->out_ports[i];
+      if (out_port->identifier.type == TYPE_EVENT &&
+          out_port->identifier.flow == FLOW_OUTPUT)
+        {
+          if (ports_connected (
+                out_port, ch->midi_out))
             {
-              if (ports_connected (
-                    ch->midi_in, in_port))
-                port_disconnect (
-                  ch->piano_roll,
-                  in_port);
+              port_disconnect (
+                out_port, ch->midi_out);
             }
         }
     }
@@ -138,10 +235,10 @@ disconnect_stereo_in_from_fader (
 {
   port_disconnect (
     ch->stereo_in->l,
-    ch->fader.stereo_in->l);
+    ch->prefader.stereo_in->l);
   port_disconnect (
     ch->stereo_in->r,
-    ch->fader.stereo_in->r);
+    ch->prefader.stereo_in->r);
 }
 
 /**
@@ -156,10 +253,10 @@ connect_stereo_in_to_fader (
 {
   port_connect (
     ch->stereo_in->l,
-    ch->fader.stereo_in->l);
+    ch->prefader.stereo_in->l, 1);
   port_connect (
     ch->stereo_in->r,
-    ch->fader.stereo_in->r);
+    ch->prefader.stereo_in->r, 1);
 }
 
 /**
@@ -234,7 +331,7 @@ connect_ch_stereo_in_to_plugin (
                 {
                   port_connect (
                     ch->stereo_in->l,
-                    in_port);
+                    in_port, 1);
                   last_index++;
                   break;
                 }
@@ -242,7 +339,7 @@ connect_ch_stereo_in_to_plugin (
                 {
                   port_connect (
                     ch->stereo_in->r,
-                    in_port);
+                    in_port, 1);
                   last_index++;
                   break;
                 }
@@ -255,7 +352,7 @@ connect_ch_stereo_in_to_plugin (
  * Connect outputs of src to inputs of dests.
  */
 static inline void
-connect_from_pl_to_pl (
+connect_pl_to_pl (
   Plugin * src,
   Plugin * dest)
 {
@@ -281,7 +378,7 @@ connect_from_pl_to_pl (
                     {
                       port_connect (
                         out_port,
-                        in_port);
+                        in_port, 1);
                       goto done1;
                     }
                 }
@@ -312,7 +409,7 @@ done1:
                     {
                       port_connect (
                         out_port,
-                        in_port);
+                        in_port, 1);
                     }
                 }
               break;
@@ -340,7 +437,7 @@ done1:
                     {
                       port_connect (
                         out_port,
-                        in_port);
+                        in_port, 1);
                       goto done2;
                     }
                 }
@@ -378,7 +475,7 @@ done2:
                     {
                       port_connect (
                         out_port,
-                        in_port);
+                        in_port, 1);
                       last_index++;
                       ports_connected++;
                       break;
@@ -390,6 +487,31 @@ done2:
             }
         }
     }
+
+  /* connect prev midi outs to next midi ins */
+  /* this connects only one midi out to all of the
+   * midi ins of the next plugin */
+  for (i = 0; i < src->num_out_ports; i++)
+    {
+      out_port = src->out_ports[i];
+
+      if (out_port->identifier.type == TYPE_EVENT)
+        {
+          for (j = 0;
+               j < dest->num_in_ports; j++)
+            {
+              in_port = dest->in_ports[j];
+
+              if (in_port->identifier.type == TYPE_EVENT)
+                {
+                  port_connect (
+                    out_port,
+                    in_port, 1);
+                }
+            }
+          break;
+        }
+    }
 }
 
 /**
@@ -415,10 +537,10 @@ connect_pl_to_fader (
             {
               port_connect (
                 out_port,
-                ch->fader.stereo_in->l);
+                ch->prefader.stereo_in->l, 1);
               port_connect (
                 out_port,
-                ch->fader.stereo_in->r);
+                ch->prefader.stereo_in->r, 1);
               break;
             }
         }
@@ -438,14 +560,14 @@ connect_pl_to_fader (
             {
               port_connect (
                 out_port,
-                ch->fader.stereo_in->l);
+                ch->prefader.stereo_in->l, 1);
               last_index++;
             }
           else if (last_index == 1)
             {
               port_connect (
                 out_port,
-                ch->fader.stereo_in->r);
+                ch->prefader.stereo_in->r, 1);
               break;
             }
         }
@@ -594,6 +716,29 @@ done2:
             }
         }
     }
+
+  /* disconnect MIDI connections */
+  for (i = 0; i < src->num_out_ports; i++)
+    {
+      out_port = src->out_ports[i];
+
+      if (out_port->identifier.type == TYPE_EVENT)
+        {
+          for (j = 0;
+               j < dest->num_in_ports; j++)
+            {
+              in_port = dest->in_ports[j];
+
+              if (in_port->identifier.type ==
+                    TYPE_EVENT)
+                {
+                  port_disconnect (
+                    out_port,
+                    in_port);
+                }
+            }
+        }
+    }
 }
 
 /**
@@ -614,13 +759,13 @@ disconnect_pl_from_fader (
       if (out_port->identifier.type == TYPE_AUDIO)
         {
           if (ports_connected (
-                out_port, ch->fader.stereo_in->l))
+                out_port, ch->prefader.stereo_in->l))
             port_disconnect (
-              out_port, ch->fader.stereo_in->l);
+              out_port, ch->prefader.stereo_in->l);
           if (ports_connected (
-                out_port, ch->fader.stereo_in->r))
+                out_port, ch->prefader.stereo_in->r))
             port_disconnect (
-              out_port, ch->fader.stereo_in->r);
+              out_port, ch->prefader.stereo_in->r);
         }
     }
 }
@@ -640,6 +785,12 @@ connect_no_prev_no_next (
    * stereo out. disconnect it */
   disconnect_stereo_in_from_fader (ch);
 
+  /* MIDI in is connected to MIDI out. disconnect
+   * it */
+  disconnect_midi_in_from_midi_out (ch);
+
+  disconnect_piano_roll_from_midi_out (ch);
+
   /* -------------------------------------------
    * connect input ports
    * ------------------------------------------- */
@@ -647,12 +798,20 @@ connect_no_prev_no_next (
   /* connect channel stereo in to plugin */
   connect_ch_stereo_in_to_plugin (ch, pl);
 
+  /* connect channel midi in to plugin */
+  connect_midi_in_to_pl (ch, pl);
+
+  connect_piano_roll_to_pl (ch, pl);
+
   /* --------------------------------------
    * connect output ports
    * ------------------------------------*/
 
   /* connect plugin to stereo out */
   connect_pl_to_fader (pl, ch);
+
+  /* connect plugin to midi out */
+  connect_pl_to_midi_out (pl, ch);
 }
 
 /**
@@ -671,6 +830,12 @@ connect_no_prev_next (
    * disconnect it */
   disconnect_ch_stereo_in_from_pl (ch, next_pl);
 
+  /* MIDI in is connected to next plugin. disconnect
+   * it */
+  disconnect_midi_in_from_pl (ch, next_pl);
+
+  disconnect_piano_roll_from_pl (ch, next_pl);
+
   /* -------------------------------------------
    * connect input ports
    * ------------------------------------------- */
@@ -678,13 +843,18 @@ connect_no_prev_next (
   /* connect channel stereo in to plugin */
   connect_ch_stereo_in_to_plugin (ch, pl);
 
+  /* connect midi in to plugin */
+  connect_midi_in_to_pl (ch, pl);
+
+  connect_piano_roll_to_pl (ch, pl);
+
   /* --------------------------------------
    * connect output ports
    * ------------------------------------*/
 
   /* connect plugin's audio outs to next
    * plugin */
-  connect_from_pl_to_pl (pl, next_pl);
+  connect_pl_to_pl (pl, next_pl);
 }
 
 /**
@@ -703,13 +873,17 @@ connect_prev_no_next (
    * disconnect it */
   disconnect_pl_from_fader (prev_pl, ch);
 
+  /* prev plugin is connected to channel midi out.
+   * disconnect it */
+  disconnect_pl_from_midi_out (prev_pl, ch);
+
   /* -------------------------------------------
    * connect input ports
    * ------------------------------------------- */
 
-  /* connect previous plugin's audio outs to
+  /* connect previous plugin's outs to
    * plugin */
-  connect_from_pl_to_pl (prev_pl, pl);
+  connect_pl_to_pl (prev_pl, pl);
 
   /* --------------------------------------
    * connect output ports
@@ -717,6 +891,9 @@ connect_prev_no_next (
 
   /* connect plugin output ports to stereo_out */
   connect_pl_to_fader (pl, ch);
+
+  /* connect plugin midi outs to midi out */
+  connect_pl_to_midi_out (pl, ch);
 }
 
 /**
@@ -742,7 +919,7 @@ connect_prev_next (
 
   /* connect previous plugin's audio outs to
    * plugin */
-  connect_from_pl_to_pl (prev_pl, pl);
+  connect_pl_to_pl (prev_pl, pl);
 
   /* ------------------------------------------
    * Connect output ports
@@ -750,7 +927,7 @@ connect_prev_next (
 
   /* connect plugin's audio outs to next
    * plugin */
-  connect_from_pl_to_pl (pl, next_pl);
+  connect_pl_to_pl (pl, next_pl);
 }
 
 /**
@@ -768,6 +945,11 @@ disconnect_no_prev_no_next (
   /* disconnect channel stereo in from plugin */
   disconnect_ch_stereo_in_from_pl (ch, pl);
 
+  /* disconnect channel midi in from plugin */
+  disconnect_midi_in_from_pl (ch, pl);
+
+  disconnect_piano_roll_from_pl (ch, pl);
+
   /* --------------------------------------
    * disconnect output ports
    * ------------------------------------*/
@@ -775,12 +957,20 @@ disconnect_no_prev_no_next (
   /* disconnect plugin from stereo out */
   disconnect_pl_from_fader (pl, ch);
 
+  /* disconnect plugin from midi out */
+  disconnect_pl_from_midi_out (pl, ch);
+
   /* -----------------------------------------
    * connect ports
    * ----------------------------------------- */
   /* channel stereo in should be connected to
    * channel stereo out. connect it */
   connect_stereo_in_to_fader (ch);
+
+  /* connect channel midi in to midi out */
+  connect_midi_in_to_midi_out (ch);
+
+  connect_piano_roll_to_midi_out (ch);
 }
 
 /**
@@ -799,11 +989,16 @@ disconnect_no_prev_next (
   /* disconnect channel stereo in from plugin */
   disconnect_ch_stereo_in_from_pl (ch, pl);
 
+  /* disconnect midi in from plugin */
+  disconnect_midi_in_from_pl (ch, pl);
+
+  disconnect_piano_roll_from_pl (ch, pl);
+
   /* --------------------------------------
    * Disconnect output ports
    * ------------------------------------*/
 
-  /* disconnect plugin's audio outs from next
+  /* disconnect plugin's midi & audio outs from next
    * plugin */
   disconnect_pl_from_pl (pl, next_pl);
 
@@ -814,6 +1009,11 @@ disconnect_no_prev_next (
    * plugin. connect it */
   connect_ch_stereo_in_to_plugin (
     ch, next_pl);
+
+  /* connect midi in to plugin */
+  connect_midi_in_to_pl (ch, next_pl);
+
+  connect_piano_roll_to_pl (ch, next_pl);
 }
 
 /**
@@ -841,12 +1041,18 @@ disconnect_prev_no_next (
    * out */
   disconnect_pl_from_fader (pl, ch);
 
+  /* disconnect plguin output from midi out */
+  disconnect_pl_from_midi_out (pl, ch);
+
   /* -----------------------------------------
    * connect ports
    * ----------------------------------------- */
   /* prev plugin should be connected to channel
    * stereo out. connect it */
   connect_pl_to_fader (prev_pl, ch);
+
+  /* connect prev plugin to midi out */
+  connect_pl_to_midi_out (prev_pl, ch);
 }
 
 /**
@@ -880,7 +1086,7 @@ disconnect_prev_next (
    * ----------------------------------------- */
   /* prev plugin should be connected to the next pl.
    * connect them */
-  connect_from_pl_to_pl (prev_pl, next_pl);
+  connect_pl_to_pl (prev_pl, next_pl);
 }
 
 /**
@@ -995,6 +1201,10 @@ channel_prepare_process (Channel * channel)
       port_clear_buffer (channel->midi_in);
       port_clear_buffer (channel->piano_roll);
     }
+  port_clear_buffer (channel->prefader.stereo_in->l);
+  port_clear_buffer (channel->prefader.stereo_in->r);
+  port_clear_buffer (channel->prefader.stereo_out->l);
+  port_clear_buffer (channel->prefader.stereo_out->r);
   port_clear_buffer (channel->fader.stereo_in->l);
   port_clear_buffer (channel->fader.stereo_in->r);
   port_clear_buffer (channel->fader.stereo_out->l);
@@ -1002,6 +1212,7 @@ channel_prepare_process (Channel * channel)
   port_clear_buffer (channel->stereo_out->l);
   port_clear_buffer (channel->stereo_out->r);
   port_clear_buffer (channel->midi_in);
+  port_clear_buffer (channel->midi_out);
   if (channel->piano_roll)
     port_clear_buffer (channel->piano_roll);
   for (j = 0; j < STRIP_SIZE; j++)
@@ -1039,6 +1250,7 @@ channel_init_loaded (Channel * ch)
   g_warn_if_fail (ch->track);
 
   /* fader */
+  ch->prefader.channel = ch;
   ch->fader.channel = ch;
 
   /* midi in / piano roll ports */
@@ -1049,11 +1261,16 @@ channel_init_loaded (Channel * ch)
   ch->midi_in->midi_events =
     midi_events_new (
       ch->midi_in);
+  ch->midi_out->midi_events =
+    midi_events_new (
+      ch->midi_out);
   ch->piano_roll->midi_events =
     midi_events_new (
       ch->piano_roll);
   ch->midi_in->identifier.owner_type =
     PORT_OWNER_TYPE_TRACK;
+  ch->midi_out->identifier.owner_type =
+    PORT_OWNER_TYPE_TRACK;
 
   /* routing */
   if (ch->output_pos > -1)
@@ -1134,12 +1351,22 @@ _create_channel (
       TYPE_EVENT,
       FLOW_INPUT,
       pll);
-  /*self->midi_in->identifier.owner_type =*/
-    /*PORT_OWNER_TYPE_TRACK;*/
   self->midi_in->midi_events =
     midi_events_new (
       self->midi_in);
   g_free (pll);
+  pll =
+    g_strdup_printf ("%s MIDI out",
+                     track->name);
+  self->midi_out =
+    port_new_with_type (
+      TYPE_EVENT,
+      FLOW_OUTPUT,
+      pll);
+  self->midi_out->midi_events =
+    midi_events_new (
+      self->midi_out);
+  g_free (pll);
   pll =
     g_strdup_printf ("%s Stereo out L",
                      track->name);
@@ -1176,6 +1403,9 @@ _create_channel (
   port_set_owner_track (
     self->midi_in,
     track);
+  port_set_owner_track (
+    self->midi_out,
+    track);
 
   /* init plugins */
   for (int i = 0; i < STRIP_SIZE; i++)
@@ -1187,6 +1417,9 @@ _create_channel (
     &self->fader,
     FADER_TYPE_CHANNEL,
     self);
+  passthrough_processor_init (
+    &self->prefader,
+    self);
 
   /* set up piano roll port */
   char * tmp =
@@ -1331,10 +1564,13 @@ channel_connect (
       ch->output_pos = -1;
       port_connect (
         ch->stereo_out->l,
-        AUDIO_ENGINE->stereo_out->l);
+        AUDIO_ENGINE->stereo_out->l, 1);
       port_connect (
         ch->stereo_out->r,
-        AUDIO_ENGINE->stereo_out->r);
+        AUDIO_ENGINE->stereo_out->r, 1);
+      port_connect (
+        ch->midi_out,
+        AUDIO_ENGINE->midi_out, 1);
     }
   else
     {
@@ -1349,29 +1585,37 @@ channel_connect (
     {
       /* connect stereo in to stereo out through
        * fader */
+      connect_stereo_in_to_fader (ch);
       port_connect (
-        ch->stereo_in->l,
-        ch->fader.stereo_in->l);
+        ch->prefader.stereo_out->l,
+        ch->fader.stereo_in->l, 1);
       port_connect (
-        ch->stereo_in->r,
-        ch->fader.stereo_in->r);
+        ch->prefader.stereo_out->r,
+        ch->fader.stereo_in->r, 1);
       port_connect (
         ch->fader.stereo_out->l,
-        ch->stereo_out->l);
+        ch->stereo_out->l, 1);
       port_connect (
         ch->fader.stereo_out->r,
-        ch->stereo_out->r);
+        ch->stereo_out->r, 1);
     }
 
+  /** Connect MIDI in and piano roll to MIDI out. */
+  connect_midi_in_to_midi_out (ch);
+  connect_piano_roll_to_midi_out (ch);
+
   if (ch->type != CT_MASTER)
     {
       /* connect channel out ports to master */
       port_connect (
         ch->stereo_out->l,
-        P_MASTER_TRACK->channel->stereo_in->l);
+        P_MASTER_TRACK->channel->stereo_in->l, 1);
       port_connect (
         ch->stereo_out->r,
-        P_MASTER_TRACK->channel->stereo_in->r);
+        P_MASTER_TRACK->channel->stereo_in->r, 1);
+      port_connect (
+        ch->midi_out,
+        P_MASTER_TRACK->channel->midi_in, 1);
     }
 }
 
@@ -1397,10 +1641,10 @@ channel_update_output (
    */
   port_connect (
     ch->stereo_out->l,
-    output->channel->stereo_in->l);
+    output->channel->stereo_in->l, 1);
   port_connect (
     ch->stereo_out->r,
-    output->channel->stereo_in->r);
+    output->channel->stereo_in->r, 1);
 
   ch->output = output;
   g_message ("setting output %s",
@@ -1469,51 +1713,11 @@ channel_get_phase (void * _channel)
   return channel->fader.phase;
 }
 
-/*void*/
-/*channel_set_volume (void * _channel, float volume)*/
-/*{*/
-  /*Channel * channel = (Channel *) _channel;*/
-  /*channel->volume = volume;*/
-  /*[> TODO update tooltip <]*/
-  /*gtk_label_set_text (channel->widget->phase_reading,*/
-                      /*g_strdup_printf ("%.1f", volume));*/
-  /*g_idle_add ((GSourceFunc) gtk_widget_queue_draw,*/
-              /*GTK_WIDGET (channel->widget->fader));*/
-/*}*/
-
-/*float*/
-/*channel_get_volume (void * _channel)*/
-/*{*/
-  /*Channel * channel = (Channel *) _channel;*/
-  /*return channel->volume;*/
-/*}*/
-
-/*static int*/
-/*redraw_fader_asnyc (Channel * channel)*/
-/*{*/
-  /*gtk_widget_queue_draw (*/
-    /*GTK_WIDGET (channel->widget->fader));*/
-
-  /*return FALSE;*/
-/*}*/
-
-
-/*static int*/
-/*redraw_pan_async (Channel * channel)*/
-/*{*/
-  /*gtk_widget_queue_draw (*/
-    /*GTK_WIDGET (channel->widget->pan));*/
-
-  /*return FALSE;*/
-/*}*/
-
 void
 channel_set_pan (void * _channel, float pan)
 {
   Channel * channel = (Channel *) _channel;
   channel->fader.pan = pan;
-  /*g_idle_add ((GSourceFunc) redraw_pan_async,*/
-              /*channel);*/
 }
 
 float
@@ -1557,8 +1761,6 @@ channel_disconnect_plugin_from_strip (
 {
   int i;
 
-  disconnect_midi_ins (ch, pos, pl);
-
   Plugin * next_plugin = NULL;
   for (i = pos + 1; i < STRIP_SIZE; i++)
     {
@@ -1740,11 +1942,6 @@ channel_add_plugin (
    * connect ports
    * ------------------------------------------ */
 
-  connect_midi_ins (
-    channel,
-    pos,
-    plugin);
-
   if (!prev_plugin && !next_plugin)
     connect_no_prev_no_next (
       channel,
@@ -1870,7 +2067,7 @@ channel_reattach_midi_editor_manual_press_port (
                 {
                   if (connect)
                     {
-                      port_connect (AUDIO_ENGINE->midi_editor_manual_press, port);
+                      port_connect (AUDIO_ENGINE->midi_editor_manual_press, port, 1);
                     }
                   else
                     {
@@ -1980,6 +2177,7 @@ channel_clone (
 
   COPY_MEMBER (type);
   clone->fader.channel = clone;
+  clone->prefader.channel = clone;
   fader_copy (&ch->fader, &clone->fader);
 
 #undef COPY_MEMBER
@@ -2042,6 +2240,7 @@ channel_disconnect (
   port_disconnect_all (channel->stereo_in->l);
   port_disconnect_all (channel->stereo_in->r);
   port_disconnect_all (channel->midi_in);
+  port_disconnect_all (channel->midi_out);
   port_disconnect_all (channel->piano_roll);
   port_disconnect_all (channel->stereo_out->l);
   port_disconnect_all (channel->stereo_out->r);
@@ -2065,6 +2264,7 @@ channel_free (Channel * channel)
   port_free (channel->stereo_in->l);
   port_free (channel->stereo_in->r);
   port_free (channel->midi_in);
+  port_free (channel->midi_out);
   port_free (channel->piano_roll);
   port_free (channel->stereo_out->l);
   port_free (channel->stereo_out->r);
@@ -2083,13 +2283,10 @@ channel_free (Channel * channel)
       /*plugin_free (pl);*/
     /*}*/
 
-  /* delete automation tracks */
+  /* remove automation tracks - they are already
+   * free'd in track_free */
   channel_remove_ats_from_automation_tracklist (
     channel);
-  for (int i = 0; i < channel->num_ats; i++)
-    {
-      automation_track_free (channel->ats[i]);
-    }
   free (channel->ats);
 
   if (channel->widget)
