@@ -67,6 +67,11 @@ engine_update_frames_per_tick (int beats_per_bar,
   /* update positions */
   transport_update_position_frames (
     &AUDIO_ENGINE->transport);
+
+  for (int i = 0; i < TRACKLIST->num_tracks; i++)
+    {
+      track_update_frames (TRACKLIST->tracks[i]);
+    }
 }
 
 static void
@@ -267,10 +272,10 @@ engine_init (
    * output */
   port_connect (
     SAMPLE_PROCESSOR->stereo_out->l,
-    self->stereo_out->l);
+    self->stereo_out->l, 1);
   port_connect (
     SAMPLE_PROCESSOR->stereo_out->r,
-    self->stereo_out->r);
+    self->stereo_out->r, 1);
 }
 
 void
@@ -394,6 +399,9 @@ engine_process_prepare (
       g_message ("pause requested handled");
       TRANSPORT->play_state = PLAYSTATE_PAUSED;
       /*zix_sem_post (&TRANSPORT->paused);*/
+      if (self->audio_backend == AUDIO_BACKEND_JACK)
+        jack_transport_stop (
+          self->client);
     }
   else if (TRANSPORT->play_state ==
            PLAYSTATE_ROLL_REQUESTED)
@@ -402,6 +410,23 @@ engine_process_prepare (
       self->remaining_latency_preroll =
         router_get_max_playback_latency (
           &MIXER->router);
+      if (self->audio_backend == AUDIO_BACKEND_JACK)
+        jack_transport_start (
+          self->client);
+    }
+
+  switch (self->audio_backend)
+    {
+    case AUDIO_BACKEND_JACK:
+#ifdef HAVE_JACK
+      engine_jack_prepare_process (self);
+#endif
+      break;
+    case AUDIO_BACKEND_ALSA:
+      engine_alsa_prepare_process (self);
+      break;
+    default:
+      break;
     }
 
   int ret =
@@ -416,6 +441,8 @@ engine_process_prepare (
 
   /* reset all buffers */
   port_clear_buffer (self->midi_in);
+  port_clear_buffer (self->midi_out);
+  port_clear_buffer (self->midi_editor_manual_press);
 
   sample_processor_prepare_process (
     &self->sample_processor, nframes);
@@ -479,8 +506,8 @@ receive_midi_events (
     {
 #ifdef HAVE_JACK
     case MIDI_BACKEND_JACK:
-      engine_jack_receive_midi_events (
-        self, nframes, print);
+      port_receive_midi_events_from_jack (
+        self->midi_in, 0, nframes);
       break;
 #endif
 #ifdef __linux__
@@ -798,6 +825,14 @@ engine_post_process (AudioEngine * self)
     {
       transport_add_to_playhead (
         TRANSPORT, self->nframes);
+      if (self->audio_backend ==
+            AUDIO_BACKEND_JACK &&
+          self->transport_type ==
+            AUDIO_ENGINE_JACK_TIMEBASE_MASTER)
+        {
+          jack_transport_locate (
+            self->client, PLAYHEAD->frames);
+        }
     }
 
   AUDIO_ENGINE->last_time_taken =
