@@ -30,10 +30,14 @@
 
 #include "project.h"
 #include "audio/channel.h"
+#ifdef HAVE_JACK
+#include "audio/engine_jack.h"
+#endif
 #include "audio/midi.h"
 #include "audio/mixer.h"
 #include "audio/modulator.h"
 #include "audio/pan.h"
+#include "audio/passthrough_processor.h"
 #include "audio/port.h"
 #include "plugins/plugin.h"
 #include "utils/arrays.h"
@@ -232,6 +236,28 @@ port_find_from_identifier (
             return ch->fader.stereo_in->r;
         }
       break;
+    case PORT_OWNER_TYPE_PREFADER:
+      tr = TRACKLIST->tracks[id->track_pos];
+      g_warn_if_fail (tr);
+      ch = tr->channel;
+      g_warn_if_fail (ch);
+      if (id->flow == FLOW_OUTPUT)
+        {
+          if (id->flags & PORT_FLAG_STEREO_L)
+            return ch->prefader.stereo_out->l;
+          else if (id->flags &
+                     PORT_FLAG_STEREO_R)
+            return ch->prefader.stereo_out->r;
+        }
+      else if (id->flow == FLOW_INPUT)
+        {
+          if (id->flags & PORT_FLAG_STEREO_L)
+            return ch->prefader.stereo_in->r;
+          else if (id->flags &
+                     PORT_FLAG_STEREO_R)
+            return ch->prefader.stereo_in->r;
+        }
+      break;
     case PORT_OWNER_TYPE_SAMPLE_PROCESSOR:
       if (id->flags & PORT_FLAG_STEREO_L)
         return SAMPLE_PROCESSOR->stereo_out->l;
@@ -328,6 +354,7 @@ port_get_all (
   _ADD (AUDIO_ENGINE->stereo_out->l);
   _ADD (AUDIO_ENGINE->stereo_out->r);
   _ADD (AUDIO_ENGINE->midi_in);
+  _ADD (AUDIO_ENGINE->midi_out);
   _ADD (AUDIO_ENGINE->midi_editor_manual_press);
   _ADD (SAMPLE_PROCESSOR->stereo_out->l);
   _ADD (SAMPLE_PROCESSOR->stereo_out->r);
@@ -350,6 +377,7 @@ port_get_all (
       _ADD (ch->stereo_out->r);
       _ADD (ch->piano_roll);
       _ADD (ch->midi_in);
+      _ADD (ch->midi_out);
 
       /* add fader ports */
       _ADD (ch->fader.stereo_in->l);
@@ -357,6 +385,12 @@ port_get_all (
       _ADD (ch->fader.stereo_out->l);
       _ADD (ch->fader.stereo_out->r);
 
+      /* add prefader ports */
+      _ADD (ch->prefader.stereo_in->l);
+      _ADD (ch->prefader.stereo_in->r);
+      _ADD (ch->prefader.stereo_out->l);
+      _ADD (ch->prefader.stereo_out->r);
+
 #define ADD_PLUGIN_PORTS \
           if (!pl) \
             continue; \
@@ -469,19 +503,53 @@ port_set_owner_fader (
 }
 
 /**
+ * Sets the owner fader & its ID.
  */
-/*void*/
-/*port_init (Port * port)*/
-/*{*/
-  /*port->id = MIXER->num_ports++;*/
-  /*port->num_destinations = 0;*/
-/*}*/
+void
+port_set_owner_prefader (
+  Port *                 port,
+  PassthroughProcessor * fader)
+{
+  g_warn_if_fail (port && fader);
+
+  port->track = fader->channel->track;
+  port->identifier.track_pos =
+    fader->channel->track->pos;
+  port->identifier.owner_type =
+    PORT_OWNER_TYPE_PREFADER;
+}
+
+/**
+ * Returns whether the Port's can be connected (if
+ * the connection will be valid and won't break the
+ * acyclicity of the graph).
+ */
+int
+ports_can_be_connected (
+  const Port * src,
+  const Port *dest)
+{
+  Graph * graph =
+    graph_new (&MIXER->router, src, dest);
+  if (graph)
+    {
+      graph_free (graph);
+      return 1;
+    }
+  else
+    return 0;
+}
 
 /**
  * Connets src to dest.
+ *
+ * @param locked Lock the connection or not.
  */
 int
-port_connect (Port * src, Port * dest)
+port_connect (
+  Port * src,
+  Port * dest,
+  const int locked)
 {
   g_warn_if_fail (src != NULL);
   g_warn_if_fail (dest != NULL);
@@ -499,6 +567,8 @@ port_connect (Port * src, Port * dest)
     &dest->identifier,
     &src->dest_ids[src->num_dests]);
   src->multipliers[src->num_dests] = 1.f;
+  src->dest_locked[src->num_dests] = locked;
+  src->dest_enabled[src->num_dests] = 1;
   src->num_dests++;
   dest->srcs[dest->num_srcs] = src;
   port_identifier_copy (
@@ -577,6 +647,9 @@ port_disconnect (Port * src, Port * dest)
   return 0;
 }
 
+/**
+ * Returns if the two ports are connected or not.
+ */
 int
 ports_connected (Port * src, Port * dest)
 {
@@ -660,6 +733,171 @@ port_apply_fader (
     }
 }
 
+void
+port_receive_midi_events_from_jack (
+  Port *      self,
+  int         start_frame,
+  int   nframes)
+{
+  if (self->internal_type !=
+        INTERNAL_JACK_PORT ||
+      self->identifier.type !=
+        TYPE_EVENT)
+    return;
+
+  void * port_buf =
+    jack_port_get_buffer (
+      JACK_PORT_T (self->data), nframes);
+  int num_events =
+    jack_midi_get_event_count (port_buf);
+
+  jack_midi_event_t jack_ev;
+  for(int i = 0; i < num_events; i++)
+    {
+      jack_midi_event_get (
+        &jack_ev, port_buf, i);
+
+      if (jack_ev.time >= start_frame &&
+          jack_ev.time < start_frame + nframes)
+        {
+          midi_events_add_event_from_buf (
+            self->midi_events,
+            jack_ev.time, jack_ev.buffer,
+            jack_ev.size);
+        }
+    }
+
+  if(self->midi_events->num_events > 0)
+    g_message ("JACK MIDI (%s): have %d events",
+               self->identifier.label,
+               num_events);
+}
+
+void
+port_receive_audio_data_from_jack (
+  Port *      port,
+  int         start_frames,
+  int   nframes)
+{
+  if (port->internal_type !=
+        INTERNAL_JACK_PORT ||
+      port->identifier.type !=
+        TYPE_AUDIO)
+    return;
+
+  float * in;
+  in =
+    (float *)
+    jack_port_get_buffer (
+      JACK_PORT_T (port->data),
+      AUDIO_ENGINE->nframes);
+
+  for (int i = start_frames;
+       i < start_frames + nframes; i++)
+    {
+      port->buf[i] +=
+        in[i];
+    }
+}
+
+void
+port_send_midi_events_to_jack (
+  Port *      port,
+  int         start_frames,
+  int   nframes)
+{
+  if (port->internal_type !=
+        INTERNAL_JACK_PORT ||
+      port->identifier.type !=
+        TYPE_EVENT)
+    return;
+
+  midi_events_copy_to_jack (
+    port->midi_events,
+    jack_port_get_buffer (
+      JACK_PORT_T (port->data),
+      AUDIO_ENGINE->nframes));
+}
+
+void
+port_send_audio_data_to_jack (
+  Port *      port,
+  int         start_frames,
+  int         nframes)
+{
+  if (port->internal_type !=
+        INTERNAL_JACK_PORT ||
+      port->identifier.type !=
+        TYPE_AUDIO)
+    return;
+
+  float * out;
+  out =
+    (float *)
+    jack_port_get_buffer (
+      JACK_PORT_T (port->data),
+      AUDIO_ENGINE->nframes);
+
+  for (int i = start_frames;
+       i < start_frames + nframes; i++)
+    {
+      out[i] = port->buf[i];
+    }
+}
+
+/**
+ * Sums the inputs coming in from JACK, before the
+ * port is processed.
+ */
+void
+port_sum_data_from_jack (
+  Port * self,
+  const int start_frame,
+  const int nframes)
+{
+  if (self->identifier.owner_type ==
+        PORT_OWNER_TYPE_BACKEND ||
+      self->internal_type !=
+        INTERNAL_JACK_PORT ||
+      self->identifier.flow !=
+        FLOW_INPUT)
+    return;
+
+  /* append events from JACK if any */
+  port_receive_midi_events_from_jack (
+    self, start_frame, nframes);
+
+  /* audio */
+  port_receive_audio_data_from_jack (
+    self, start_frame, nframes);
+}
+
+/**
+ * Sends the port data to JACK, after the port
+ * is processed.
+ */
+void
+port_send_data_to_jack (
+  Port * self,
+  const int start_frame,
+  const int nframes)
+{
+  if (self->identifier.owner_type ==
+        PORT_OWNER_TYPE_BACKEND ||
+      self->internal_type !=
+        INTERNAL_JACK_PORT ||
+      self->identifier.flow !=
+        FLOW_OUTPUT)
+    return;
+
+  /* send midi events */
+  port_send_midi_events_to_jack (
+    self, start_frame, nframes);
+
+  /* send audio data */
+  port_send_audio_data_to_jack (
+    self, start_frame, nframes);
+}
 
 /**
  * First sets port buf to 0, then sums the given
@@ -682,7 +920,8 @@ port_sum_signal_from_inputs (
   int k, l;
 
   g_warn_if_fail (
-    start_frame + nframes <= AUDIO_ENGINE->nframes);
+    start_frame + nframes <=
+    AUDIO_ENGINE->nframes);
 
   switch (port->identifier.type)
     {
@@ -690,6 +929,9 @@ port_sum_signal_from_inputs (
       if (noroll)
         break;
 
+      port_sum_data_from_jack (
+        port, start_frame, nframes);
+
       for (k = 0; k < port->num_srcs; k++)
         {
           src_port = port->srcs[k];
@@ -702,6 +944,9 @@ port_sum_signal_from_inputs (
             nframes, 0);
         }
 
+      port_send_data_to_jack (
+        port, start_frame, nframes);
+
       /* send UI notification */
       if (port->midi_events->num_events > 0)
         {
@@ -729,6 +974,9 @@ port_sum_signal_from_inputs (
           break;
         }
 
+      port_sum_data_from_jack (
+        port, start_frame, nframes);
+
       for (k = 0; k < port->num_srcs; k++)
         {
           src_port = port->srcs[k];
@@ -742,6 +990,9 @@ port_sum_signal_from_inputs (
               port->buf[l] += src_port->buf[l];
             }
         }
+
+      port_send_data_to_jack (
+        port, start_frame, nframes);
       break;
     case TYPE_CONTROL:
       {
@@ -798,6 +1049,93 @@ port_sum_signal_from_inputs (
     }
 }
 
+#ifdef HAVE_JACK
+/**
+ * Sets whether to expose the port to JACk and
+ * exposes it or removes it from JACK.
+ */
+void
+port_set_expose_to_jack (
+  Port * self,
+  int    expose)
+{
+  enum JackPortFlags flags;
+  if (self->identifier.flow == FLOW_INPUT)
+    flags = JackPortIsInput;
+  else if (self->identifier.flow == FLOW_OUTPUT)
+    flags = JackPortIsOutput;
+  else
+    g_return_if_reached ();
+
+  char * type = NULL;
+  if (self->identifier.type == TYPE_AUDIO)
+    type = JACK_DEFAULT_AUDIO_TYPE;
+  else if (self->identifier.type == TYPE_EVENT)
+    type = JACK_DEFAULT_MIDI_TYPE;
+  else
+    g_return_if_reached ();
+
+  if (expose)
+    {
+      char * label =
+        port_get_full_designation (self);
+      self->data =
+        (void *) jack_port_register (
+          AUDIO_ENGINE->client,
+          label,
+          type, flags, 0);
+      g_free (label);
+      self->internal_type = INTERNAL_JACK_PORT;
+    }
+  else
+    {
+      jack_port_unregister (
+        AUDIO_ENGINE->client,
+        JACK_PORT_T (self->data));
+    }
+}
+#endif
+
+/**
+ * Returns a full designation of the port in the
+ * format "Track/Port" or "Track/Plugin/Port".
+ *
+ * Must be free'd.
+ */
+char *
+port_get_full_designation (
+  Port * self)
+{
+  PortIdentifier * id = &self->identifier;
+
+  switch (id->owner_type)
+    {
+    case PORT_OWNER_TYPE_BACKEND:
+    case PORT_OWNER_TYPE_SAMPLE_PROCESSOR:
+      return g_strdup (id->label);
+      break;
+    case PORT_OWNER_TYPE_PLUGIN:
+      return
+        g_strdup_printf (
+          "%s/%s/%s",
+          self->plugin->track->name,
+          self->plugin->descr->name,
+          id->label);
+      break;
+    case PORT_OWNER_TYPE_TRACK:
+    case PORT_OWNER_TYPE_PREFADER:
+    case PORT_OWNER_TYPE_FADER:
+      return
+        g_strdup_printf (
+          "%s/%s",
+          self->track->name,
+          id->label);
+      break;
+    default:
+      g_return_val_if_reached (NULL);
+    }
+}
+
 /**
  * Prints all connections.
  */
