@@ -34,6 +34,7 @@
 #include "audio/midi.h"
 #include "audio/modulator.h"
 #include "audio/pan.h"
+#include "audio/passthrough_processor.h"
 #include "audio/port.h"
 #include "audio/routing.h"
 #include "audio/track.h"
@@ -159,6 +160,12 @@ get_node_name (
           "%s Fader",
           node->fader->channel->track->name);
       break;
+    case ROUTE_NODE_TYPE_PREFADER:
+      return
+        g_strdup_printf (
+          "%s Pre-Fader",
+          node->prefader->channel->track->name);
+      break;
     case ROUTE_NODE_TYPE_SAMPLE_PROCESSOR:
       return
         g_strdup ("Sample Processor");
@@ -293,6 +300,11 @@ node_process (
       fader_process (
         node->fader, local_offset, nframes);
     }
+  else if (node->type == ROUTE_NODE_TYPE_PREFADER)
+    {
+      passthrough_processor_process (
+        node->prefader, local_offset, nframes);
+    }
   else if (node->type ==
            ROUTE_NODE_TYPE_SAMPLE_PROCESSOR)
     {
@@ -346,18 +358,17 @@ node_process (
               midi_events_dequeue (
                 port->midi_events);
               if (port->midi_events->num_events > 0)
-                g_message ("piano roll port %s has %d events",
-                           port->identifier.label,
-                           port->midi_events->num_events);
+                g_message (
+                  "piano roll port %s has %d events",
+                  port->identifier.label,
+                  port->midi_events->num_events);
             }
-
         }
 
       /* if midi editor manual press */
       else if (port == AUDIO_ENGINE->
             midi_editor_manual_press)
         {
-          port_clear_buffer (port);
           midi_events_dequeue (
             AUDIO_ENGINE->
               midi_editor_manual_press->
@@ -365,9 +376,11 @@ node_process (
         }
 
       /* if channel stereo in */
-      else if (port->identifier.type == TYPE_AUDIO &&
+      else if (
+          port->identifier.type == TYPE_AUDIO &&
           port->identifier.flow == FLOW_INPUT &&
-          port->track)
+          port->identifier.owner_type ==
+            PORT_OWNER_TYPE_TRACK)
         {
           chan = port->track->channel;
 
@@ -430,8 +443,9 @@ node_process (
         }
 
       /* if JACK stereo out */
-      else if (port->identifier.owner_type ==
-               PORT_OWNER_TYPE_BACKEND &&
+      else if (
+          port->identifier.owner_type ==
+            PORT_OWNER_TYPE_BACKEND &&
           port->identifier.type == TYPE_AUDIO &&
           port->identifier.flow == FLOW_OUTPUT)
         {
@@ -475,24 +489,36 @@ node_process (
                 case AUDIO_BACKEND_ALSA:
 #ifdef __linux__
                   /* write interleaved */
-                  if (port ==
-                        AUDIO_ENGINE->stereo_out->l)
-                  for (i = local_offset;
-                       i < local_offset + nframes;
-                       i++)
-                      {
-                        AUDIO_ENGINE->alsa_out_buf[i * 2] =
-                          port->srcs[0]->buf[i];
-                      }
-                  else if (port ==
-                        AUDIO_ENGINE->stereo_out->r)
-                  for (i = local_offset;
-                       i < local_offset + nframes;
-                       i++)
-                      {
-                        AUDIO_ENGINE->alsa_out_buf[i * 2 + 1] =
-                          port->srcs[0]->buf[i];
-                      }
+                  for (j = 0; j < port->num_srcs;
+                       j++)
+                    {
+                      if (port ==
+                            AUDIO_ENGINE->
+                              stereo_out->l)
+                      for (i = local_offset;
+                           i < local_offset +
+                             nframes;
+                           i++)
+                          {
+                            AUDIO_ENGINE->
+                              alsa_out_buf[i * 2] +=
+                                port->
+                                  srcs[j]->buf[i];
+                          }
+                      else if (port ==
+                            AUDIO_ENGINE->
+                              stereo_out->r)
+                      for (i = local_offset;
+                           i < local_offset +
+                             nframes;
+                           i++)
+                          {
+                            AUDIO_ENGINE->
+                              alsa_out_buf[
+                                i * 2 + 1] +=
+                              port->srcs[j]->buf[i];
+                          }
+                    }
 #endif
                   break;
                 case AUDIO_BACKEND_PORT_AUDIO:
@@ -519,6 +545,42 @@ node_process (
             }
         }
 
+      /* if JACK MIDI out */
+      else if (port->identifier.owner_type ==
+               PORT_OWNER_TYPE_BACKEND &&
+          port->identifier.type == TYPE_EVENT &&
+          port->identifier.flow == FLOW_OUTPUT)
+        {
+          if (!AUDIO_ENGINE->exporting)
+            {
+              /*float * out;*/
+              /*int ret;*/
+              switch (AUDIO_ENGINE->audio_backend)
+                {
+                case AUDIO_BACKEND_JACK:
+#ifdef HAVE_JACK
+                  /* by this time, the Master
+                   * channel should have its
+                   * MIDI out port filled.
+                   * pass its buffer to JACK */
+                  port_sum_signal_from_inputs (
+                    port, local_offset,
+                    nframes, noroll);
+
+                  midi_events_copy_to_jack (
+                    port->midi_events,
+                    jack_port_get_buffer (
+                      JACK_PORT_T (port->data),
+                      AUDIO_ENGINE->nframes));
+#endif
+                  break;
+                default:
+                  /* TODO */
+                  break;
+                }
+            }
+        }
+
       else
         {
           port_sum_signal_from_inputs (
@@ -843,7 +905,7 @@ node_connect (
 static GraphNode *
 find_node_from_port (
   Graph * graph,
-  Port * port)
+  const Port * port)
 {
   GraphNode * node;
   for (int i = 0; i < graph->n_graph_nodes; i++)
@@ -888,6 +950,22 @@ find_node_from_fader (
   return NULL;
 }
 
+static GraphNode *
+find_node_from_prefader (
+  Graph * graph,
+  PassthroughProcessor * prefader)
+{
+  GraphNode * node;
+  for (int i = 0; i < graph->n_graph_nodes; i++)
+    {
+      node = graph->graph_nodes[i];
+      if (node->type == ROUTE_NODE_TYPE_PREFADER &&
+          node->prefader == prefader)
+        return node;
+    }
+  return NULL;
+}
+
 static GraphNode *
 find_node_from_sample_processor (
   Graph * graph,
@@ -923,6 +1001,8 @@ graph_node_new (
     node->port = (Port *) data;
   else if (type == ROUTE_NODE_TYPE_FADER)
     node->fader = (Fader *) data;
+  else if (type == ROUTE_NODE_TYPE_PREFADER)
+    node->prefader = (PassthroughProcessor *) data;
   else if (type ==
              ROUTE_NODE_TYPE_SAMPLE_PROCESSOR)
     node->sample_processor =
@@ -1001,7 +1081,10 @@ node_free (
   free (node);
 }
 
-static inline void
+/**
+ * Frees the graph and its members.
+ */
+void
 graph_free (
   Graph * self)
 {
@@ -1199,7 +1282,8 @@ add_port (
   Graph * self,
   Port *   port)
 {
-  PortOwnerType owner = port->identifier.owner_type;
+  PortOwnerType owner =
+    port->identifier.owner_type;
 
   /*add_port_node (self, port);*/
   if (port->num_dests == 0 &&
@@ -1216,7 +1300,8 @@ add_port (
   else if (port->num_dests == 0 &&
            port->num_srcs == 0 &&
            owner != PORT_OWNER_TYPE_PLUGIN &&
-           owner != PORT_OWNER_TYPE_FADER)
+           owner != PORT_OWNER_TYPE_FADER &&
+           owner != PORT_OWNER_TYPE_PREFADER)
     {
     }
   else if (port->num_srcs == 0 &&
@@ -1224,6 +1309,8 @@ add_port (
         port->identifier.flow == FLOW_OUTPUT) &&
       !(owner == PORT_OWNER_TYPE_FADER &&
         port->identifier.flow == FLOW_OUTPUT) &&
+      !(owner == PORT_OWNER_TYPE_PREFADER &&
+        port->identifier.flow == FLOW_OUTPUT) &&
       !(owner ==
           PORT_OWNER_TYPE_SAMPLE_PROCESSOR &&
         port->identifier.flow == FLOW_OUTPUT))
@@ -1237,6 +1324,9 @@ add_port (
            !(owner == PORT_OWNER_TYPE_FADER &&
              port->identifier.flow ==
                FLOW_INPUT) &&
+           !(owner == PORT_OWNER_TYPE_PREFADER &&
+             port->identifier.flow ==
+               FLOW_INPUT) &&
            !(owner ==
                PORT_OWNER_TYPE_SAMPLE_PROCESSOR &&
              port->identifier.flow == FLOW_INPUT))
@@ -1311,7 +1401,7 @@ set_node_playback_latency (
   GraphNode * node,
   long        dest_latency)
 {
-  long max_latency = 0, parent_latency;
+  /*long max_latency = 0, parent_latency;*/
 
   /* set this node's latency */
   node->playback_latency =
@@ -1332,9 +1422,86 @@ set_node_playback_latency (
     }
 }
 
+/**
+ * Checks for cycles in the graph.
+ *
+ * @return 1 if valid, 0 if invalid.
+ */
+static int
+graph_is_valid (
+  Graph * self)
+{
+  int i;
+
+  /* empty list to put sorted elements */
+  /*GraphNode * sorted_nodes[self->n_graph_nodes];*/
+  /*int num_sorted_nodes = 0;*/
+
+  /* set of all nodes without incoming edges */
+  /*GraphNode * trigger_nodes[self->n_init_triggers];*/
+  /*int num_trigger_nodes = 0;*/
+  /*for (i = 0; i < self->n_init_triggers; i++)*/
+    /*{*/
+      /*trigger_nodes[num_trigger_nodes++] =*/
+        /*self->init_trigger_list[i];*/
+    /*}*/
+
+  GraphNode * n, * m;
+  while (self->n_init_triggers > 0)
+    {
+      n =
+        self->init_trigger_list[
+          --self->n_init_triggers];
+      /*g_message ("init trigger %d %s",*/
+                 /*self->n_init_triggers,*/
+                 /*get_node_name (n));*/
+      /*sorted_nodes[num_sorted_nodes++] = n;*/
+      for (i = n->n_childnodes - 1;
+           i >= 0; i--)
+        {
+          m = n->childnodes[i];
+          /*g_message ("child node %d %s",*/
+                     /*i,*/
+                     /*get_node_name (m));*/
+          n->n_childnodes--;
+          m->init_refcount--;
+          if (m->init_refcount == 0)
+            {
+              self->init_trigger_list[
+                self->n_init_triggers++] = m;
+            }
+        }
+    }
+
+  for (i = 0; i < self->n_graph_nodes; i++)
+    {
+      n = self->graph_nodes[i];
+      if (n->n_childnodes > 0 ||
+          n->init_refcount > 0)
+        {
+          graph_print (self);
+          /*g_warning ("graph invalid");*/
+          return 0;
+        }
+    }
+
+  return 1;
+}
+
+/**
+ * Returns a new graph, or NULL if the graph is
+ * invalid.
+ *
+ * Optionally adds a new connection for the given
+ * src and dest ports.
+ *
+ * Should be used every time the graph is changed.
+ */
 Graph *
 graph_new (
-  Router * router)
+  Router * router,
+  const Port *   src,
+  const Port *   dest)
 {
   int i, j, k;
   GraphNode * node, * node2;
@@ -1366,6 +1533,11 @@ graph_new (
         self, ROUTE_NODE_TYPE_FADER,
         &tr->channel->fader);
 
+      /* add the prefader */
+      graph_add_node ( \
+        self, ROUTE_NODE_TYPE_PREFADER,
+        &tr->channel->prefader);
+
 #define ADD_PLUGIN \
           if (!pl || pl->deleting) \
             continue; \
@@ -1441,6 +1613,7 @@ graph_new (
   node_connect (node, node2);
 
   Fader * fader;
+  PassthroughProcessor * prefader;
   for (int i = 0; i < TRACKLIST->num_tracks; i++)
     {
       tr = TRACKLIST->tracks[i];
@@ -1448,6 +1621,7 @@ graph_new (
         continue;
 
       fader = &tr->channel->fader;
+      prefader = &tr->channel->prefader;
 
       /* connect the fader */
       node =
@@ -1470,6 +1644,27 @@ graph_new (
         find_node_from_port (self, port);
       node_connect (node, node2);
 
+      /* connect the prefader */
+      node =
+        find_node_from_prefader (
+          self, prefader);
+      port = prefader->stereo_in->l;
+      node2 =
+        find_node_from_port (self, port);
+      node_connect (node2, node);
+      port = prefader->stereo_in->r;
+      node2 =
+        find_node_from_port (self, port);
+      node_connect (node2, node);
+      port = prefader->stereo_out->l;
+      node2 =
+        find_node_from_port (self, port);
+      node_connect (node, node2);
+      port = prefader->stereo_out->r;
+      node2 =
+        find_node_from_port (self, port);
+      node_connect (node, node2);
+
 #define CONNECT_PLUGIN \
           if (!pl || pl->deleting) \
             continue; \
@@ -1524,6 +1719,26 @@ graph_new (
         self, port);
     }
 
+  /* connect the src/dest if not NULL */
+  /* this code is only for creating graphs to test
+   * if the connection between src->dest is valid */
+  if (src && dest)
+    {
+      node =
+        find_node_from_port (self, src);
+      node2 =
+        find_node_from_port (self, dest);
+      node_connect (node, node2);
+
+      if (graph_is_valid (self))
+        return self;
+      else
+        {
+          graph_free (self);
+          return NULL;
+        }
+    }
+
   /* ========================
    * calculate latencies of each port and each
    * processor
@@ -1560,7 +1775,7 @@ router_init (
   Router * self)
 {
   /* create the initial graph */
-  self->graph1 = graph_new (self);
+  self->graph1 = graph_new (self, NULL, NULL);
 
   self->graph2 = NULL;
 }
