@@ -30,6 +30,7 @@
 #include "audio/group_track.h"
 #include "audio/instrument_track.h"
 #include "audio/master_track.h"
+#include "audio/midi_track.h"
 #include "audio/instrument_track.h"
 #include "audio/track.h"
 #include "gui/backend/events.h"
@@ -155,6 +156,10 @@ track_new (
       group_track_init (track);
       ct = CT_BUS;
       break;
+    case TRACK_TYPE_MIDI:
+      midi_track_init (track);
+      ct = CT_MIDI;
+      break;
     case TRACK_TYPE_CHORD:
       break;
     default:
@@ -260,13 +265,13 @@ track_set_recording (Track *   track,
     {
       port_connect (
         AUDIO_ENGINE->stereo_in->l,
-        channel->stereo_in->l);
+        channel->stereo_in->l, 1);
       port_connect (
         AUDIO_ENGINE->stereo_in->r,
-        channel->stereo_in->r);
+        channel->stereo_in->r, 1);
       port_connect (
         AUDIO_ENGINE->midi_in,
-        channel->midi_in);
+        channel->midi_in, 1);
     }
   else
     {
@@ -753,18 +758,19 @@ track_get_automation_tracklist (Track * track)
   switch (track->type)
     {
     case TRACK_TYPE_CHORD:
+    case TRACK_TYPE_MARKER:
       break;
     case TRACK_TYPE_BUS:
     case TRACK_TYPE_GROUP:
     case TRACK_TYPE_INSTRUMENT:
     case TRACK_TYPE_AUDIO:
     case TRACK_TYPE_MASTER:
+    case TRACK_TYPE_MIDI:
         {
-          ChannelTrack * bt = (ChannelTrack *) track;
-          return &bt->automation_tracklist;
+          return &track->automation_tracklist;
         }
     default:
-      /* TODO */
+      g_warn_if_reached ();
       break;
     }
 
@@ -816,6 +822,7 @@ track_get_channel (Track * track)
     case TRACK_TYPE_AUDIO:
     case TRACK_TYPE_BUS:
     case TRACK_TYPE_GROUP:
+    case TRACK_TYPE_MIDI:
       return track->channel;
     default:
       g_return_val_if_reached (NULL);
@@ -882,6 +889,9 @@ track_stringize_type (
     case TRACK_TYPE_AUDIO:
       return g_strdup (
         _("Audio"));
+    case TRACK_TYPE_MIDI:
+      return g_strdup (
+        _("MIDI"));
     case TRACK_TYPE_BUS:
       return g_strdup (
         _("Bus"));
@@ -894,12 +904,48 @@ track_stringize_type (
     case TRACK_TYPE_GROUP:
       return g_strdup (
         _("Group"));
+    case TRACK_TYPE_MARKER:
+      return g_strdup (
+        _("Group"));
     default:
       g_warn_if_reached ();
       return NULL;
     }
 }
 
+/**
+ * Updates the frames of each position in each child
+ * of the track recursively.
+ */
+void
+track_update_frames (
+  Track * self)
+{
+  int i;
+  for (i = 0; i < self->num_lanes; i++)
+    {
+      track_lane_update_frames (self->lanes[i]);
+    }
+  for (i = 0; i < self->num_chord_regions; i++)
+    {
+      region_update_frames (
+        self->chord_regions[i]);
+    }
+  for (i = 0; i < self->num_scales; i++)
+    {
+      scale_object_update_frames (
+        self->scales[i]);
+    }
+  for (i = 0; i < self->num_markers; i++)
+    {
+      marker_update_frames (
+        self->markers[i]);
+    }
+
+  automation_tracklist_update_frames (
+    &self->automation_tracklist);
+}
+
 /**
  * Wrapper to get the track name.
  */
