@@ -256,6 +256,8 @@ main_window_widget_init (MainWindowWidget * self)
       activate_create_ins_track },
     { "create-bus-track",
       activate_create_bus_track },
+    { "create-midi-track",
+      activate_create_midi_track },
     { "create-group-track",
       activate_create_group_track },
     { "delete-selected-tracks",
@@ -282,6 +284,14 @@ main_window_widget_init (MainWindowWidget * self)
     { "goto-next-marker",
       activate_goto_next_marker },
 
+    /* transport - jack */
+    { "set-timebase-master",
+      activate_set_timebase_master },
+    { "set-transport-client",
+      activate_set_transport_client },
+    { "unlink-jack-transport",
+      activate_unlink_jack_transport },
+
     /* tracks */
     { "delete-selected-tracks",
       activate_delete_selected_tracks },
