@@ -41,6 +41,7 @@
 #include "gui/widgets/marker_track.h"
 #include "gui/widgets/master_track.h"
 #include "gui/widgets/midi_activity_bar.h"
+#include "gui/widgets/midi_track.h"
 #include "gui/widgets/timeline_arranger.h"
 #include "gui/widgets/timeline_bg.h"
 #include "gui/widgets/track.h"
@@ -426,7 +427,9 @@ show_context_menu (TrackWidget * self)
     {
       char * str;
 
-      if (track->type != TRACK_TYPE_MASTER)
+      if (track->type != TRACK_TYPE_MASTER &&
+          track->type != TRACK_TYPE_CHORD &&
+          track->type != TRACK_TYPE_MARKER)
         {
           /* delete track */
           if (num_selected == 1)
@@ -586,6 +589,7 @@ track_widget_new (Track * track)
     NEW_TW (INSTRUMENT, instrument);
     NEW_TW (MASTER, master);
     NEW_TW (AUDIO, audio);
+    NEW_TW (MIDI, midi);
     NEW_TW (MARKER, marker);
     default:
       break;
